"eMap1P" <-
function(me, mm1, mm2=NULL, ptype, output.tag, n.permute=10, 
  p.cut=1e-4, cis.only=FALSE, cis.distance=1e6, eChr=0, ePos=0,
  mChr=0, mPos=0, trace.it=TRUE, tol=1e-7, nna.percent=0.75,
  np.max=100000, np=c(100, 1000, 5000, 10000, 50000),
  aim.p=c(0.1, 0.05, 0.02, 0.01, 0.002), confidence.p=0.0001,
  permute.grp=NULL)
{
  if(is.vector(me)){
    me = matrix(me, nrow=1)
  }
  ncol.me = ncol(me)
  nrow.me = nrow(me)

  if(is.vector(mm1)){
    mm1 = matrix(mm1, nrow=1)
  }
  ncol.mm = ncol(mm1)
  nrow.mm = nrow(mm1)
  
  if(!is.null(mm2)){
    if(is.vector(mm2)){
      mm2 = matrix(mm2, nrow=1)
    }else{
      if(ncol.mm != ncol(mm1) || nrow.mm != nrow(mm1))
      stop("mm2 and mm1 have different dimensions\n")
    }
  }
    
  if(ncol.mm != ncol.me){
      stop("mm and me have different number of columns!\n")
  }

  if(!is.null(permute.grp)){
    if(length(permute.grp) != ncol.me){
      stop("length of permute.grp does not match me!\n")
    }
  }
  
  if(cis.only){
    if(!is.numeric(eChr) || length(eChr) != nrow.me){
      stop(sprintf("eChr must be a numeric vector of length %d\n", nrow.me))
    }
    if(!is.numeric(ePos) || length(ePos) != nrow.me){
      stop(sprintf("ePos must be a numeric vector of length %d\n", nrow.me))
    }
    if(!is.numeric(mChr) || length(mChr) != nrow.mm){
      stop(sprintf("mChr must be a numeric vector of length %d\n", nrow.mm))
    }
    if(!is.numeric(mPos) || length(mPos) != nrow.mm){
      stop(sprintf("mPos must be a numeric vector of length %d\n", nrow.mm))
    }
  }

  dims = numeric(5)
  dims[1] = nrow.me
  dims[2] = nrow.mm
  dims[3] = ncol.me
  dims[4] = as.numeric(!is.null(mm2))
  dims[5] = n.permute

  succeed = 0
  
  mm1.na = is.na(mm1)
  mm1[mm1.na] = 0
  
  if(is.null(mm2)){
    mm2 = 0.0
    mm2.na = 1
  }else{
    mm2.na = is.na(mm2)
    mm2[mm2.na] = 0
  }
    
  if(ptype==1){
    # ---------------------------------------------------------
    # if permutation type (ptype) == 1
    #
    # repeated computation for each permuted gene exprssion data,
    # results of each permuted gene expression data are written
    # into *_eql_per_k.txt and *eqtl_freq_per_k.txt 
    #
    # n.permute and pcut are only used when ptype==1
    # ---------------------------------------------------------
    
    for(p in 1:n.permute){
      cat("permutation", p, date(), "\n")
      
      if(is.null(permute.grp)){
        topermute = sample(ncol.me)
      }else{
        topermute = numeric(ncol.me)
        for(grp in unique(permute.grp)){
          ww = which(permute.grp==grp)
          topermute[ww] = sample(ww)
        }
      }
      me.p = me[,topermute,drop=FALSE]
      me.na = is.na(me.p)
      me.p[me.na] = 0
      
      output    = character(2)
      output[1] = sprintf("%s_eqtl_per_%d.txt", output.tag, p)
      output[2] = sprintf("%s_freq_per_%d.txt", output.tag, p)
      
      Z = .C("lmEQTL", as.integer(dims), as.double(me.p), as.double(mm1), 
              as.double(mm2), as.integer(!me.na), as.integer(!mm1.na), 
              as.integer(!mm2.na), as.character(output), as.double(p.cut),
              as.integer(cis.only), as.integer(cis.distance),
              as.integer(eChr), as.integer(ePos),
              as.integer(mChr), as.integer(mPos), 
              as.double(tol), as.double(nna.percent), 
              as.integer(succeed), PACKAGE="eMap")
    }
    mm1[mm1.na] = NA
    if(!is.null(mm2)){ mm2[mm2.na] = NA }
    
    return(Z[[length(Z)]])
    
  }else if(ptype==2){
    # ---------------------------------------------------------
    # if permutation type == 2
    #
    # for each gene, evaluate the pemrutation p-value of its
    # most significant nominal p-value
    #
    # when type==2, only np.max, np, and aim.p are used.
    # ---------------------------------------------------------

    if(length(np) != length(aim.p)){
      stop("the length of np must equal to the length of aim.p\n")
    }

    if(any(diff(np)<=0)){ stop("np must be strictly ascending\n") }

    if(any(diff(aim.p)>=0)){ stop("aim.p must be strictly descending\n") }

    if(np.max <= max(np)){
      stop("np.max must be bigger than max(np)\n")
    }

    me.na = is.na(me)
    me[me.na] = 0

    cuts  = qbinom(1-confidence.p, np, aim.p)
    pcuts = cuts/np
    
    # np:  total number of permutations for each targeted p-value
    # nnp: number of additional permutations for each targeted p-value
    np  = c(np, np.max)
    nnp = c(np[1], diff(np))

    idum = round(runif(1, -1000000, -1))
    
    # ---------------------------------------------------------
    # variables for group-wise permutations
    #
    # Rgrp:  group-wise permutation or not
    # Rngrp: the number of groups
    # Rgrpn: sample size of each permutation group
    # Rgrps: the samples in each permutation group
    # ---------------------------------------------------------
    
    if(is.null(permute.grp)){
      Rgrp  = Rgrpn = Rgrps = 0
      Rngrp = 1
    }else{
      Rgrp  = 1                           
      Rngrp = length(unique(permute.grp)) 
      Rgrpn = numeric(Rngrp)              
      Rgrps = matrix(0, nrow=Rngrp, ncol=ncol.me)
      
      i=0
      for(grp in unique(permute.grp)){
        i  = i + 1
        ww = which(permute.grp==grp)
        Rgrpn[i] = length(ww)
        Rgrps[i,1:Rgrpn[i]] = ww - 1
      }
    }
        
    # ---------------------------------------------------------
    # first carry out "nnp[1]" permutations. Record the best 
    # associated markers and corresponding nominal/permuted 
    # p-value for eah gene.
    #
    # pval:  nominal p-value
    # ppval: permutation p-value
    # ---------------------------------------------------------
    
    # the number of permutatios to do
    dims[5] = nnp[1]
    
    best.m = pval = ppval = numeric(nrow.me)
    
    Z = .C("lmEQTL_permute", as.integer(dims), as.double(me), as.double(mm1), 
            as.double(mm2), as.integer(!me.na), as.integer(!mm1.na), 
            as.integer(!mm2.na), bestM = as.integer(best.m), 
            pval=as.double(pval), perP=as.double(ppval), as.integer(cis.only), 
            as.integer(cis.distance), as.integer(eChr), as.integer(ePos), 
            as.integer(mChr), as.integer(mPos), as.integer(trace.it),
            as.double(tol), as.double(nna.percent), as.integer(idum),
            as.integer(Rgrp), as.integer(Rngrp), as.integer(Rgrpn),
            as.integer(Rgrps), as.integer(succeed), PACKAGE="eMap")

    best.m = Z[["bestM"]]
    pval   = Z[["pval"]]
    ppval  = Z[["perP"]]
    
    # number of permutations for each gene
    npall  = rep(nnp[1], dims[1])
    
    # ---------------------------------------------------------
    # continue the permutations.
    # note that the last elements of np is the maximum number
    # of permutations to do. length(pcuts) = length(np) - 1
    # ---------------------------------------------------------
    
    for(i in 2:length(np)){
      which.kp = which(ppval < pcuts[i-1])
      if(length(which.kp)==0){break}

      me1    = me[which.kp,,drop=FALSE]
      me1.na = me.na[which.kp,,drop=FALSE]
      
      # update the number of genes
      dims[1] = nrow(me1)

      # update the number of permutations to do
      dims[5] = nnp[i] 

      # update the chromosome information
      if(cis.only){
        eChr1 = eChr[which.kp]
        ePos1 = ePos[which.kp]
      }else{
        eChr1 = 0
        ePos1 = 0
      }
      
      bbm = pvs = ppvs = numeric(dims[1])
      
      Z = .C("lmEQTL_permute", as.integer(dims), as.double(me1), as.double(mm1),
              as.double(mm2), as.integer(!me1.na), as.integer(!mm1.na),
              as.integer(!mm2.na), bestM = as.integer(bbm),
              pval=as.double(pvs), perP=as.double(ppvs), as.integer(cis.only), 
              as.integer(cis.distance), as.integer(eChr1), as.integer(ePos1),
              as.integer(mChr), as.integer(mPos), as.integer(trace.it),
              as.double(tol), as.double(nna.percent), as.integer(idum),
              as.integer(Rgrp), as.integer(Rngrp), as.integer(Rgrpn),
              as.integer(Rgrps), as.integer(succeed), PACKAGE="eMap")
      
      # ---------------------------------------------------------
      # update permutation p-values
      # w1 and w2 are weights for the previous permutations and  
      # the current permutations
      # ---------------------------------------------------------

      w2 = nnp[i]/np[i]
      w1 = 1 - w2
      ppval[which.kp] = w1*ppval[which.kp] + w2*Z[["perP"]]
      npall[which.kp] = np[i]
    }

    me[me.na] = NA
    mm1[mm1.na] = NA
    if(!is.null(mm2)){ mm2[mm2.na] = NA }

    gID = 1:nrow.me
    mID = best.m+1
    dataF = data.frame(geneID=gID, markerID=mID, pval=pval,
      permuteP=ppval, npermute=npall)
    return(dataF)
  }else{
    stop("ptype can only be 1 or 2\n")
  }
}
